CREATE TABLE `user_clients` (
  `id` int unsigned AUTO_INCREMENT NOT NULL UNIQUE,
  `name` varchar(128) NOT NULL UNIQUE,
  `created` datetime NOT NULL,
  `modified` datetime NOT NULL,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB;

CREATE TABLE `users` (
  `id` int unsigned AUTO_INCREMENT NOT NULL UNIQUE,
  `username` char(16) NOT NULL UNIQUE,
  `name` varchar(128) NOT NULL,
  `email_address` varchar(64) NOT NULL,
  `phone_number` char(32) NOT NULL,
  `password` char(40) NOT NULL,
  `active_account` boolean NOT NULL,
  `campaign_admin` boolean NOT NULL,
  `created` datetime NOT NULL,
  `modified` datetime NOT NULL,
  `user_client_id` int unsigned NOT NULL,
  PRIMARY KEY (`id`),
  FOREIGN KEY (`user_client_id`) REFERENCES `user_clients`(`id`)
	ON DELETE CASCADE
) ENGINE=InnoDB;

CREATE TABLE `campaigns` (
  `id` int unsigned AUTO_INCREMENT NOT NULL UNIQUE,
  `name` varchar(128) NOT NULL UNIQUE,
  `orderer` varchar(128) NOT NULL,
  `start_time` date NOT NULL,
  `end_time` date,
  `message_subject` varchar(128) NOT NULL,
  `message` text NOT NULL,
  `target_amount` smallint unsigned NOT NULL,
  `accomplished_amount` smallint unsigned NOT NULL,
  `created` datetime NOT NULL,
  `modified` datetime NOT NULL,
  `user_client_id` int unsigned NOT NULL,
  PRIMARY KEY (`id`),
  FOREIGN KEY (`user_client_id`) REFERENCES `user_clients`(`id`)
	ON DELETE CASCADE
) ENGINE=InnoDB;

CREATE TABLE `campaigns_users` (
  `id` int unsigned AUTO_INCREMENT NOT NULL UNIQUE,
  `user_id` int unsigned NOT NULL,
  `campaign_id` int unsigned NOT NULL,
  `phone_number` char(32) NOT NULL,
  `email_address` varchar(64) NOT NULL,
  `target_amount` smallint unsigned NOT NULL,
  `accomplished_amount` smallint unsigned NOT NULL,
  `calls_per_day` smallint unsigned NOT NULL,
  `calls_per_day_suggestion` smallint unsigned NOT NULL,
  `created` datetime NOT NULL,
  `modified` datetime NOT NULL,
  PRIMARY KEY (`id`),
  FOREIGN KEY (`user_id`) REFERENCES `users`(`id`) ON DELETE CASCADE,
  FOREIGN KEY (`campaign_id`) REFERENCES `campaigns`(`id`) ON DELETE CASCADE,
  CONSTRAINT UNIQUE (`user_id`, `campaign_id`)
) ENGINE=InnoDB;

CREATE TABLE `message_links` (
  `id` int unsigned AUTO_INCREMENT NOT NULL UNIQUE,
  `target_url` text NOT NULL,
  `target_url_text` text NOT NULL,
  `beacon` boolean NOT NULL,
  `created` datetime NOT NULL,
  `modified` datetime NOT NULL,
  `campaign_id` int unsigned NOT NULL,
  PRIMARY KEY (`id`),
  FOREIGN KEY (`campaign_id`) REFERENCES `campaigns`(`id`)
    ON DELETE CASCADE
) ENGINE=InnoDB;

CREATE TABLE `sales_arguments` (
  `id` int unsigned AUTO_INCREMENT NOT NULL UNIQUE,
  `argument` varchar(128) NOT NULL,
  `created` datetime NOT NULL,
  `modified` datetime NOT NULL,
  `campaign_id` int unsigned NOT NULL,
  PRIMARY KEY (`id`),
  FOREIGN KEY (`campaign_id`) REFERENCES `campaigns`(`id`)
    ON DELETE CASCADE,
  CONSTRAINT UNIQUE (`argument`, `campaign_id`)
) ENGINE=InnoDB;

CREATE TABLE `target_client_states` (
  `id` char(32) NOT NULL UNIQUE,
  `order` smallint NOT NULL UNIQUE,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB;

CREATE TABLE `target_clients` (
  `id` int unsigned AUTO_INCREMENT NOT NULL UNIQUE,
  `name` varchar(128) NOT NULL,
  `y_id` char(16) NOT NULL,
  `domain` varchar(128) NOT NULL,
  `exchange_number` char(32) NOT NULL,
  `address` text NOT NULL,
  `email_address` varchar(64) NOT NULL,
  `homepage` varchar(128) NOT NULL,
  `inoa_id` int unsigned,
  `created` datetime NOT NULL,
  `modified` datetime NOT NULL,
  `target_client_state_id` char(32) NOT NULL,
  `campaign_id` int unsigned NOT NULL,
  `campaigns_user_id` int unsigned,
  PRIMARY KEY (`id`),
  FOREIGN KEY (`campaign_id`) REFERENCES `campaigns`(`id`)
    ON DELETE CASCADE,
  FOREIGN KEY (`target_client_state_id`) REFERENCES `target_client_states`(`id`)
    ON DELETE CASCADE,
  FOREIGN KEY (`campaigns_user_id`) REFERENCES `campaigns_users`(`id`)
    ON DELETE SET NULL,
  CONSTRAINT UNIQUE (`name`, `campaign_id`)
) ENGINE=InnoDB;

CREATE TABLE `classifications` (
  `id` int unsigned AUTO_INCREMENT NOT NULL UNIQUE,
  `classification` varchar(64) NOT NULL,
  `created` datetime NOT NULL,
  `modified` datetime NOT NULL,
  `campaign_id` int unsigned NOT NULL,
  PRIMARY KEY (`id`),
  FOREIGN KEY (`campaign_id`) REFERENCES `campaigns`(`id`)
    ON DELETE CASCADE,
  CONSTRAINT UNIQUE (`classification`, `campaign_id`)
) ENGINE=InnoDB;

CREATE TABLE `classifications_target_clients` (
  `target_client_id` int unsigned NOT NULL,
  `classification_id` int unsigned NOT NULL,
  PRIMARY KEY (`target_client_id`, `classification_id`),
  FOREIGN KEY (`target_client_id`) REFERENCES `target_clients`(`id`)
    ON DELETE CASCADE,
  FOREIGN KEY (`classification_id`) REFERENCES `classifications`(`id`)
    ON DELETE CASCADE
) ENGINE=InnoDB;

CREATE TABLE `event_types` (
  `id` char(32) NOT NULL UNIQUE,
  `priority` smallint NOT NULL,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB;

CREATE TABLE `event_type_classes` (
  `id` char(32) NOT NULL UNIQUE,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB;
INSERT INTO `event_type_classes` (`id`) VALUES ('Soitto');
INSERT INTO `event_type_classes` (`id`) VALUES ('Viesti');
INSERT INTO `event_type_classes` (`id`) VALUES ('Tietomuutos');
INSERT INTO `event_type_classes` (`id`) VALUES ('Valittavissa');
INSERT INTO `event_type_classes` (`id`) VALUES ('Ulkopuolinen');

CREATE TABLE `event_type_classes_event_types` (
  `event_type_id` char(32) NOT NULL,
  `event_type_class_id` char(32) NOT NULL,
  `order` smallint NOT NULL,
  PRIMARY KEY (`event_type_id`, `event_type_class_id`),
  FOREIGN KEY (`event_type_id`) REFERENCES `event_types`(`id`)
    ON DELETE CASCADE,
  FOREIGN KEY (`event_type_class_id`) REFERENCES `event_type_classes`(`id`)
    ON DELETE CASCADE
) ENGINE=InnoDB;

CREATE TABLE `reason_no_interests` (
  `id` varchar(128) NOT NULL UNIQUE,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB;

CREATE TABLE `events` (
  `id` int unsigned AUTO_INCREMENT NOT NULL UNIQUE,
  `start_time` datetime NOT NULL,
  `end_time` datetime NOT NULL,
  `contact_at` datetime,
  `interest` smallint NOT NULL,
  `comment` text NOT NULL,
  `contact_name` varchar(128) NOT NULL,
  `contact_title` varchar(32) NOT NULL,
  `contact_phone_number` varchar(32) NOT NULL,
  `contact_email_address` varchar(64) NOT NULL,
  `created` datetime NOT NULL,
  `modified` datetime NOT NULL,
  `event_type_id` char(32) NOT NULL,
  `target_client_id` int unsigned NOT NULL,
  `user_id` int unsigned NOT NULL,
  `sales_argument_id` int unsigned,
  `reason_no_interest_id` varchar(128),
  PRIMARY KEY (`id`),
  FOREIGN KEY (`event_type_id`) REFERENCES `event_types`(`id`),
  FOREIGN KEY (`target_client_id`) REFERENCES `target_clients`(`id`),
  FOREIGN KEY (`user_id`) REFERENCES `users`(`id`)
    ON DELETE CASCADE,
  FOREIGN KEY (`sales_argument_id`) REFERENCES `sales_arguments`(`id`),
  FOREIGN KEY (`reason_no_interest_id`) REFERENCES `reason_no_interests`(`id`)
) ENGINE=InnoDB;

CREATE VIEW `latest_events` AS (
  SELECT *
    FROM `events`
    WHERE `end_time` IN (
      SELECT max(`end_time`)
        FROM `events` AS `latest`
        WHERE `latest`.`target_client_id` = `events`.`target_client_id`
          AND `latest`.`event_type_id` NOT IN (
            SELECT `event_type_id`
              FROM `event_type_classes_event_types`
              WHERE `event_type_class_id` = 'Ulkopuolinen'
          )
    )
);

CREATE TABLE `events_message_links` (
  `id` char(40) NOT NULL UNIQUE,
  `event_count` smallint unsigned NOT NULL,
  `message_link_id` int unsigned NOT NULL,
  `user_event_id` int unsigned NOT NULL,
  `target_client_event_id` int unsigned,
  PRIMARY KEY (`id`),
  FOREIGN KEY (`message_link_id`) REFERENCES `message_links`(`id`)
    ON DELETE CASCADE,
  FOREIGN KEY (`user_event_id`) REFERENCES `events`(`id`)
    ON DELETE CASCADE,
  FOREIGN KEY (`target_client_event_id`) REFERENCES `events`(`id`)
    ON DELETE CASCADE,
  CONSTRAINT UNIQUE (`message_link_id`, `user_event_id`),
  CONSTRAINT UNIQUE (`message_link_id`, `target_client_event_id`)
) ENGINE=InnoDB;

CREATE TABLE `profile_classes` (
  `id` int unsigned AUTO_INCREMENT NOT NULL UNIQUE,
  `name` varchar(32) NOT NULL UNIQUE,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB;

CREATE TABLE `profile_classes_users` (
  `user_id` int unsigned NOT NULL,
  `profile_class_id` int unsigned NOT NULL,
  `profile_value` smallint unsigned NOT NULL,
  `created` datetime NOT NULL,
  `modified` datetime NOT NULL,
  PRIMARY KEY (`user_id`, `profile_class_id`),
  FOREIGN KEY (`user_id`) REFERENCES `users`(`id`)
    ON DELETE CASCADE,
  FOREIGN KEY (`profile_class_id`) REFERENCES `profile_classes`(`id`)
    ON DELETE CASCADE
) ENGINE=InnoDB;

CREATE TABLE `encouragements` (
  `id` int unsigned AUTO_INCREMENT NOT NULL UNIQUE,
  -- encouragement tulisi olla UNIQUE, mutta tyypin vuoksi ei ole  
  `encouragement` text NOT NULL,
  `created` datetime NOT NULL,
  `modified` datetime NOT NULL,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB;

CREATE TABLE `encouragements_profile_classes` (
  `encouragement_id` int unsigned NOT NULL,
  `profile_class_id` int unsigned NOT NULL,
  PRIMARY KEY (`encouragement_id`, `profile_class_id`),
  FOREIGN KEY (`encouragement_id`) REFERENCES `encouragements`(`id`)
    ON DELETE CASCADE,
  FOREIGN KEY (`profile_class_id`) REFERENCES `profile_classes`(`id`)
    ON DELETE CASCADE
) ENGINE=InnoDB;

CREATE VIEW `event_types_with_classes` AS (
  SELECT `priority`, `event_type_id`, `event_type_class_id`, `order`
    FROM `event_types`
      INNER JOIN `event_type_classes_event_types`
        ON `event_type_classes_event_types`.`event_type_id` = `event_types`.`id`
      INNER JOIN `event_type_classes`
        ON `event_type_classes`.`id` = `event_type_classes_event_types`.`event_type_class_id`
);
